ASMB,A,B,L
*
* 13210/7900 bootext that boots HP-IPL/OS
* Configured for slots 16/17 - self-patches from sw bits 6-11
* Last code mod 2/22/07  Converted to EXTASMB 10/2/07
*
* The boot loader loads 2011-16010 (octal) from cylinder 0
* and JSB's to address contained in location 2055.
* After being started by the boot-rom this code relocates the
* first 256 words of itself to 76011 and runs from there to
* load in HP-IPL/OS (or any 31K binary starting at 2) from disk.
* The binary is loaded to the range of 2-76001 from the 8K mark
* on the disk.. cylinder 1 head 0 sector 16 on a 7900.
* When booting set SR bit 5 to halt to configure TTY slot, and
* set SR bits 3,4 to drive number. SR bits 3-5 clear for normal.
* Note.. "normal" bootroms can only boot drive 0, to "boot" drive 1
* a modified bootloader program/rom/etc is required, it should set
* SR 3,4 to tell the boot extension which drive to load from.
* 
* This code contains a routine that writes this bootext to
* cylinder 0, run from 2012 with SR bits 0,1=drive#,
* SR bits 6-11=first disk controller slot,
* SR bit 15=0 for removeable, 1 for fixed platter.
* A separate utility or HP-IPL/OS itself is required to write
* the 2-76001 boot image to cyl 1 sect 16 and on.
*
* Much of this code was copied from the 13210A-90003 manual.
*
           ORG 2011B
DC         EQU 16B
CC         EQU 17B
           HLT 33B               error!
           JMP WBEXT             ENTRY TO LAY BOOTEXT TO DISK
           JMP HIENT             ENTRY TO RUN UP HIGH AND BOOT
CW1A       OCT 120016            DMA PROGRAM CONTROL WORD (DC=16)
CW2B       OCT 100002            DMA STARTING ADDRESS
CW3A       OCT 176000            DMA LENGTH -1024 DEC
RDCMD      OCT 020000            READ COMMAND
SKCMD      OCT 030000            SEEK COMMAND
STCMD      OCT 000000            STATUS CHECK COMMAND
CYLNO      OCT 000001            CYLINDER NUMBER
HDSEC      OCT 000020            HEAD/SECTOR = 16 DEC, 1020 (oct) FOR FIXED
* HDSEC bits 8,9 = head, bits 0-4 = sector
DN         OCT 0                 DRIVE NUMBER
C1000      OCT 1000              MASK FOR BIT 9
C400       OCT 400               MASK FOR BIT 8
C3         OCT 3                 MASK FOR BITS 0,1
CSLEN      OCT 177741            COMPLEMENTARY "SECTOR" LENGTH = -31
SESIZ      OCT 2000              "SECTOR" SIZE = 1KW
RSINC      DEC 8                 NUMBER OF REAL SECTORS IN A "SECTOR"
* note.. real disk sectors = 128 words.. hpiplos sectors = 1K
* will dma in 1K at a time, bumping start adr by CSSIZ and sector by RSINC
WRAP1      OCT 30                WRAP HDSEC 30 TO 400
WRAP2      OCT 430               WRAP HDSEC 430 TO 0 AND INC CYLNO
WRAPM      OCT 437               100011111 HEAD/SECTOR MASK (IGNORE B9)
*
ADR1       OCT 2011              LOADED ADDRESS
ADR2       OCT 76011             DESTINATION ADDRESS
CCSIZ      OCT 177400            COMP. COPY SIZE - 256 WORDS
BADR       OCT 76013             ADDRESS TO JUMP TO BOOT
*
RELOC      JSB PATCH             patch slots to sr bits 6-11
RLOOP      LDA ADR1,I            GET LOCATION
           STA ADR2,I            STORE UP HIGH
           ISZ ADR1              BUMP
           ISZ ADR2              POINTERS
           ISZ CCSIZ             MORE TO COPY?
           JMP RLOOP             NO. COPY MORE
           JMP BADR,I            START RUNNING UP HIGH
*
           ORG 2055B
           DEF *+1
           NOP                   BOOT-ROM JSB ENTRY
           JMP RELOC             RELOCATE TO 76011
*
HIENT      LIA 1                 get sw reg
           AND C30               isolate drive# bits 3,4
           ARS,ARS
           ARS                   shift right 3 times
           STA DN                save as drive number
*
KLOOP      JSB SEEK              MOVE HEAD TO DESIRED CYLINDER
PD1        CLC DC                ENSURE CONTROL SIGNALS ARE RESET ON
PC1        CLC CC                DATA AND COMMAND CHANNELS
           LDA CW1A              GET FIRST DMA CONTROL WORD
           OTA 6                 OUTPUT TO DMA CHANNEL
           CLC 2                 PREPARE DMA FOR DMA CONTROL WORD 2
           LDA CW2B              GET DMA CONTROL WORD 2
           OTA 2                 OUTPUT TO DMA CHANNEL
           STC 2                 PREPARE DMA FOR DMA CONTROL WORD 3
           LDA CW3A              GET DMA CONTROL WORD 3
           OTA 2                 OUTPUT TO DMA CHANNEL
           LDA RDCMD             GET INTERFACE READ COMMAND WORD
           IOR DN                OR WITH DRIVE NUMBER
PC2        OTA CC                OUTPUT TO COMMAND CHANNEL
PD2        STC DC,C              PREPARE DATA CHANNEL FOR READ OPERATION
           STC 6,C               ACTIVATE DMA
PC3        STC CC,C              INITIATE READ OPERATION
PC4        SFS CC                IS READ OPERATION COMPLETE
           JMP *-1               NO. WAIT.
*           CLF 6
*PD3        CLC DC
*PC5        CLC CC
*           JSB WASTE
*           JSB STATS             CHECK STATUS
           ISZ CSLEN             YES. CHECK FOR NEXT SECTOR.
           JMP NXTSE             MORE. JUMP TO NEXT SECTOR.
           CLF 6                 DMA OFF
*           JMP *+1,I             FAR JUMP TO LOC. 2
*           OCT 2
*           HLT 44B
           LIA 1                 get sr settings
           AND C40               isolate bit 5
           SZA                   skip if A is zero
           JMP CFGH              jump to config halt if bit 5 of SR set
           JMP *+1,I             FAR JUMP TO LOC. 2
           OCT 2
CFGH       LDA 355B              config halt - get current TTY
           OTA 1                 display on switches
           HLT 77B               halt 77 - now select SR and modify
           JMP *+1,I             far jump to 70 to autoconfig
           OCT 70
*WASTE      NOP
*           JMP *+1               ensure drive signals toggle...
*           JMP *+1               I'm reaching....
*           JMP *+1               I'm reaching....
*           JMP WASTE,I
NXTSE      JSB INCSE             CALL SUB TO SET UP FOR NEXT SECTOR
           JMP KLOOP             LOOP TO LOAD NEXT SECTOR
*
C30        OCT 30                mask for drive bits 3,4
C40        OCT 40                mask for bit 5
*
INCSE      NOP                   SUB ENTRY
           LDA CW2B              GET DMA ADDRESS
           ADA SESIZ             ADD 2000, 1K "SECTOR"
           STA CW2B              WRITE IT BACK
           LDA HDSEC             GET HEAD/SECTOR
           ADA RSINC             ADD 8 TO GET TO NEXT "SECTOR"
           STA HDSEC             WRITE IT BACK
           AND WRAPM
           CPA WRAP1             CHECK FOR WRAP TO ODD HEAD
           JMP WRODD             YES. JUMP
           CPA WRAP2             CHECK FOR WRAP TO NEXT CYLINDER
           JMP WRCYL             YES. JUMP
           JMP INCSE,I           RETURN
WRODD      LDA HDSEC             WRAP ODD - GET HEAD/SECTOR
           AND C1000             KEEP BIT 9, REST 0
           IOR C400              MAKE x400
           STA HDSEC             WRITE IT BACK
           JMP INCSE,I           RETURN
WRCYL      LDA HDSEC             WRAP CYLINDER - GET HEAD/SECTOR
           AND C1000             KEEP BIT 9, REST 0
           STA HDSEC             WRITE IT BACK
           ISZ CYLNO             INCREMENT CYLINDER
           JMP INCSE,I           RETURN
*
SEEK       NOP                   SUBROUTINE ENTRY
PD4        CLC DC                RESET CONTROL SIGNALS
PC6        CLC CC
           LDA CYLNO             GET CYLINDER ADDRESS
PD5        OTA DC
PD6        STC DC,C              OUTPUT CYL ADDR TO DATA CHANNEL
           LDA SKCMD             GET SEEK COMMAND WORD
           IOR DN                COMBINE WITH DRIVE NUMBER
PC7        OTA CC                OUTPUT SEEK COMMAND TO COMMAND CHANNEL
PC8        STC CC,C              INITIATE SEEK COMMAND
PD7        SFS DC                FIRST ADDRESS WORD ACCEPTED?
           JMP *-1               NO. WAIT.
           LDA HDSEC             YES GET HEAD/SECTOR ADDRESS WORD
PD8        OTA DC
PD9        STC DC,C              OUTPUT HEAD/SECTOR ADDRESS TO DATA CHANNEL
PC9        SFS CC                SEEK OPERATION COMPLETE?
           JMP *-1               NO. WAIT.
*           JSB STATS             YES. GO CHECK STATUS
           JMP SEEK,I            RETURN
*
*STATS      NOP
*PD10       CLC DC
*PC10       CLC CC
*           LDA STCMD             GET STATUS CHECK COMMAND WORD
*           IOR DN                COMBINE WITH DRIVE NUMBER
*PC11       OTA CC                OUTPUT TO COMMAND CHANNEL
*PC12       STC CC,C              INITIATE STATUS CHECK COMMAND
*PD11       SFS DC                IS STATUS WORD READY?
*           JMP *-1               NO. WAIT.
*PD12       LIA DC                LOAD STATUS WORD
*           SLA                   IS ANY ERROR BIT SET?
*           JMP ERR               YES. ERROR.
*           JMP STATS,I           RETURN
*
*ERR        HLT 55B               ERROR - HALT WITH CODE 55
*
* Patch subroutine
* slot in bits 6-11 of switch register
PATCH      NOP
           LIA 1            get switch settings
           AND C7700        keep just slot bits
           SZA,RSS          skip if not zero
           JMP PATCH,I      exit sub if clear (no patch)           
           ARS,ARS
           ARS,ARS
           ARS,ARS          shift A right 6 times
           STA CHAN         save in chan
           LDA PLADR        get patch list adr
           STA PPTR         save in patch pointer
PLOOP      LDB PPTR,I       get address to patch
           SZB,RSS          skip if not zero
           JMP PATC2        if zero jump to patch CC locs
           LDA 1,I          get instruction
           AND C1777        zero low 6 bits
           IOR CHAN         or with slot#
           STA 1,I          put it back
           ISZ PPTR         increment patch ptr
           JMP PLOOP        loop to patch next location
           HLT 66B          should never get here
PATC2      ISZ CHAN         next slot for CC
           LDA PLAD2        get patch list adr
           STA PPTR         save in patch pointer
PLOO2      LDB PPTR,I       get address to patch
           SZB,RSS          skip if not zero
           JMP PATCH,I      if zero exit sub, done
           LDA 1,I          get instruction
           AND C1777        zero low 6 bits
           IOR CHAN         or with slot#
           STA 1,I          put it back
           ISZ PPTR         increment patch ptr
           JMP PLOO2        loop to patch next location
           HLT 66B          should never get here
C7700      OCT 7700         mask for bits 6-11 (slot)
CHAN       OCT 0
PPTR       OCT 0
C1777      OCT 177700
PLADR      DEF *+1          list of points to patch...
           DEF CW1A
           DEF CW1W
           DEF PD1
           DEF PD2
*           DEF PD3
           DEF PD4
           DEF PD5
           DEF PD6
           DEF PD7
           DEF PD8
           DEF PD9
*           DEF PD10
*           DEF PD11
*           DEF PD12
           DEF PD13
           DEF PD14
           DEF PD15
           DEF PD16
           DEF PD17
           DEF PD18
           DEF PD19
           DEF PD20
           DEF PD21
           OCT 0
PLAD2      DEF *+1
           DEF PC1
           DEF PC2
           DEF PC3
           DEF PC4
*           DEF PC5
           DEF PC6
           DEF PC7
           DEF PC8
           DEF PC9
*           DEF PC10
*           DEF PC11
*           DEF PC12
           DEF PC13
           DEF PC14
           DEF PC15
           DEF PC16
           DEF PC17
           DEF PC18
           DEF PC19
           DEF PC20
           OCT 0
*
* INITIALIZATION CODE - WRITE BOOTEXT TO CYLINDER 0
* SR BITS 0,1 = DRIVE NUMBER
* SR BITS 6-11 = DC SLOT#
* SR BIT 15 = SET FOR FIXED PLATTER
*
CW1W       OCT 100016            DMA CONTROL WORD FOR WRITE, SLOT 16
CW2        OCT 002011            START ADDRESS (WRITE)
CW3        OCT 164000            -6144 IN OCTAL
WRCMD      OCT 010000            WRITE COMMAND WORD
HDSE2      OCT 000000
WBEXT      JSB PATCH             patch if SR 6-11 <> 0
           LIA 1
           AND C3
           STA DN                SET DRIVE NUMBER
           LIA 1                 get switches
           SSA,RSS               skip if bit 15 set
           JMP DOWRT             if bit 15 clear do write
           LDA C1000             bit 9 set
           STA HDSE2             SET BIT 9 IF FIXED PLATTER
           LDA HDSEC
           IOR HDSE2
           STA HDSEC             SET BIT 9 IN HDSEC TOO IF FIXED
DOWRT      CLC 0                 TURN OFF INTERRUPTS IN CASE ON
PD13       CLC DC                RESET CONTROL SIGNALS
PC13       CLC CC
           CLA                   CLEAR A
PD14       OTA DC                CYLINDER = 0
PD15       STC DC,C              OUTPUT CYL ADDR TO DATA CHANNEL
           LDA SKCMD             GET SEEK COMMAND WORD
           IOR DN                COMBINE WITH DRIVE NUMBER
PC14       OTA CC                OUTPUT SEEK COMMAND TO COMMAND CHANNEL
PC15       STC CC,C              INITIATE SEEK COMMAND
PD16       SFS DC                FIRST ADDRESS WORD ACCEPTED?
           JMP *-1               NO. WAIT.
           LDA HDSE2             HDSEC = 0 OR 1000 FOR FIXED
PD17       OTA DC                
PD18       STC DC,C              OUTPUT HEAD/SECTOR ADDRESS TO DATA CHANNEL
PC16       SFS CC                SEEK OPERATION COMPLETE?
           JMP *-1               NO. WAIT.
PD19       CLC DC
PC17       CLC CC
           LDA CW1W              GET FIRST DMA CONTROL WORD
           OTA 6                 OUTPUT TO DMA CHANNEL
           CLC 2                 PREPARE FOR CONTROL WORD 2
           LDA CW2               GET WRITE START ADDRESS
           OTA 2                 OUTPUT TO DMA CHANNEL
           STC 2                 PREPARE FOR CONTROL WORD 3
           LDA CW3               GET COMP. LENGTH
           OTA 2                 OUTPUT TO DMA CHANNE:
PD20       STC DC
PD21       STF DC                ENABLE DMA TO DATA CHANNEL DATA TRANSFER
           STC 6,C               ACTIVATE DMA
           LDA WRCMD             GET INTERFACE WRITE COMMAND
           IOR DN
PC18       OTA CC                OUTPUT COMMAND TO COMMAND CHANNEL
PC19       STC CC,C              INITIATE WRITE OPERATION
PC20       SFS CC                IS WRITE COMPLETE?
           JMP *-1               NO. WAIT.
           CLF 6                 DMA OFF
*           JSB STATS             CHECK STATUS
           HLT 77B               HALT WITH CODE 77 - SUCCESS
           ASC 8, 2/22/07 WTN    
* END OF SOURCE
           END
